import { useState } from 'react';
import { useQuery, useMutation } from '@apollo/client';
import { GET_GROUPS, GET_TASKS, GET_STEP } from '../graphql/queries';
import { PRINT_TASK, PRINT_STEP, CREATE_NOTE } from '../graphql/mutations';
import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';

interface GraphQLStep {
  id: number;
  name: string;
  instructions: string;
  taskId: number;
  order: number;
  notes: GraphQLNote[];
  printCount: number;
  lastPrintedAt: string | null;
}

interface GraphQLNote {
  id: number;
  content: string;
  createdBy: number;
  user?: {
    id: number;
    name: string;
    createdAt: Date;
    updatedAt: Date;
  };
}

interface GraphQLTask {
  id: number;
  name: string;
  groupId: number;
  steps: GraphQLStep[];
  notes: GraphQLNote[];
  printCount: number;
  lastPrintedAt: string | null;
}

interface GraphQLGroup {
  id: number;
  name: string;
  tasks: GraphQLTask[];
}

function toStepWithNotes(step: GraphQLStep): StepWithNotes {
  return {
    ...step,
    notes: (step.notes || []).map((note: GraphQLNote) => ({ ...note })),
    printCount: step.printCount ?? 0,
    lastPrintedAt: step.lastPrintedAt ?? null,
  };
}

function toTaskWithSteps(task: GraphQLTask): TaskWithSteps {
  return {
    ...task,
    steps: (task.steps || []).map(toStepWithNotes),
    notes: (task.notes || []),
    printCount: task.printCount ?? 0,
    lastPrintedAt: task.lastPrintedAt ?? null,
  };
}

function toGroupWithTasks(group: GraphQLGroup): GroupWithTasks {
  return {
    ...group,
    tasks: (group.tasks || []).map(toTaskWithSteps),
  };
}

export function useTaskData() {
  const [selectedGroup, setSelectedGroup] = useState<GroupWithTasks>();
  const [selectedTask, setSelectedTask] = useState<TaskWithSteps>();
  const [selectedStep, setSelectedStep] = useState<StepWithNotes>();

  const { data: groupsData, loading: groupsLoading } = useQuery(GET_GROUPS);
  const { data: tasksData, loading: tasksLoading } = useQuery(GET_TASKS, {
    variables: { groupId: selectedGroup?.id },
    skip: !selectedGroup,
  });
  const { data: stepData, loading: stepLoading } = useQuery(GET_STEP, {
    variables: { id: selectedStep?.id },
    skip: !selectedStep,
  });

  const [printTask] = useMutation(PRINT_TASK);
  const [printStep] = useMutation(PRINT_STEP);
  const [createNote] = useMutation(CREATE_NOTE);

  const handlePrintTask = async (taskId: string, userId: string) => {
    try {
      await printTask({ variables: { id: taskId, userId } });
    } catch (error) {
      console.error('Error printing task:', error);
    }
  };

  const handlePrintStep = async (stepId: string, userId: string) => {
    try {
      await printStep({ variables: { id: stepId, userId } });
    } catch (error) {
      console.error('Error printing step:', error);
    }
  };

  const handleAddNote = async (content: string, userId: string) => {
    if (!selectedStep) return;

    try {
      await createNote({
        variables: {
          content,
          stepId: selectedStep.id,
          userId,
        },
      });
    } catch (error) {
      console.error('Error adding note:', error);
    }
  };

  return {
    groups: (groupsData?.groups || []).map(toGroupWithTasks),
    tasks: (tasksData?.tasks || []).map(toTaskWithSteps),
    step: stepData?.step ? toStepWithNotes(stepData.step) : undefined,
    loading: groupsLoading || tasksLoading || stepLoading,
    selectedGroup,
    selectedTask,
    selectedStep,
    setSelectedGroup,
    setSelectedTask,
    setSelectedStep,
    handlePrintTask,
    handlePrintStep,
    handleAddNote,
  };
}